#!/usr/bin/env python3

import sys
import logging
import socket
import socketserver
from pathlib import Path
from subprocess import check_output
import shlex
import os

logging.basicConfig(level=logging.INFO)

CONFIG = None


class RequestHandler(socketserver.BaseRequestHandler):
    def setup(self) -> None:
        logging.info("Start request.")

    def handle(self) -> None:
        conn = self.request
        while True:
            data = conn.recv(1024).decode("utf-8")

            if not data:
                break

            data = shlex.split(data)

            logging.info(f"recv: {data!r}")
            if command_allowed(data):
                resp = check_output()
                conn.sendall(resp)

            else:
                logging.error("Invalid request %r", data)

    def finish(self) -> None:
        logging.info("Finish request.")


class Server(socketserver.ThreadingUnixStreamServer):
    def server_activate(self) -> None:
        logging.info("Server started on %s", self.server_address)
        super().server_activate()


if __name__ == "__main__":
    socket_path = Path(sys.argv[1])

    if socket_path.exists():
        socket_path.unlink()

    with Server(str(socket_path), RequestHandler) as server:
        server.serve_forever()
